"use strict";
this.name	     = "Engine_Sound_FG";
this.version     = "1.2";    // Date 2015-12-09
this.description = "Creates an engine sound that follows the speed of the your ship, thruster sounds when maneuvering, and some additional sounds.";
this.author	     = "Fritz G.";   
this.copyright	 = "© 2015 Fritz G.";
this.licence	 = "CC BY-NC-SA 4.0";

//------------------------------------------------------------------------------------------------------------
// Credits:
// This OXP is based on ideas developed in http://oolite.aegidian.org/bb/viewtopic.php?f=2&t=17920
// and on a demonstration OXP by Norby. An updated version of Norby's OXP is available under 
// http://wiki.alioth.net/index.php/Engine_Sound. The sound files were taken from BGS OXP by Pagroove, 
// PhantorGorth, Thargoid, Tricky & Svengali. It is available under http://wiki.alioth.net/index.php/BGS.
// The fan sound files were made from a file from freesound.org where it was uploaded by suzzy457 and  
// licensed under CC BY 3.0 (https://freesound.org/people/suzzy457/sounds/268146).
// 
// Differences to Engine Sounds OXP by Norby (apart from the additional sounds):
// This OXP uses the same (or very similar) 25 sound files as Engine Sounds but choses them in relation 
// speed/maxSpeed. Norby's OXP uses the absolute value for speed for file selection, so slower ships will 
// sound differently at top speed than faster ships. With this OXP, all ships sound the same but can
// use the full "spectrum" of the available sound files.
//
// BGS OXP compatibility:
// BGS uses a constant ambient engine sound when speed > 0 (using the sound file that is played at 25 
// different speeds in this and Norby's OXP). So in BGS you can't hear speed differences, and this 
// actually started the forum discussion mentioned above. BGS uses two additional sound files to create
// accelerating and deceleration noises - the files are used in this OXP for the torus drive.
// BGS and this OXP can be used together, but if the BGS ambient sounds are left on, not only the crowd
// noise and the radio chatter will be played but also a (constant) engine sound. The "double engine" 
// doesn't sound bad but is of course a little bit louder, so it could be a good idea to increase the    
// maneuvering and perhaps the standby sound volume (BGS turns off its engine sound completely on standstill).  
// If chatter and crowds noise have been disabled in BGS, this OXP will turn off the BGS engine sound 
// even if $es_BGSoff is set to false.
//------------------------------------------------------------------------------------------------------------
// configuration (note: volume should be between 0 and 1, values above about 0.5 will probably be too loud here)
this.$es_volume           = 0.30;	// volume for engine and torus sounds 
this.$es_volumeStandby    = 0.15;	// engine volume for speed = 0, should be less than $es_volume
this.$es_volumeManeuver   = 0.20;	// volume for maneuvering sound 
this.$es_torusSound       = true;	// change to false to disable the torus starting and stopping sounds
this.$es_maneuveringSound = 20;		// sound file used for maneuvering sounds (1..25), set 0 to disable.
this.$es_shutOff          = true;   // shut-off sound when docking
this.$es_volumeFan        = 0.15;	// volume for cooling fan, set 0 to disable
this.$es_fanStartTemp     = 0.27;   // temperature that will start the cooling fan
this.$es_BGSoff           = false; 	// turn off BGS ambient sounds (will turn off chatter and crowd noise too)
//------------------------------------------------------------------------------------------------------------

// internal properties for engine sounds 
this.$es_soundsEngines = [];         // array for storing the different sounds
this.$es_soundSourceEngines = null;  
this.$es_soundPlaying  = 1;          // (1 .. 26, 1 = standby, 26 = torus)

// internal properties for maneuvering sounds 
this.$es_soundSourceManeuver = null;   
this.$es_oldOrientation = null;            // stores the ship's orientation

// internal properties for torus sounds 
this.$es_soundsTorus = [];           // array for storing the different sounds 
this.$es_soundSourceTorus = null;   
this.$es_torusOn = false;

// internal properties for witch drive sounds (not ready for release yet)
// this.$es_soundsWitch = [];           // array for storing the different sounds 
// this.$es_soundSourceWitch = null;   

// internal properties for cooling fan sounds 
this.$es_soundsFan = [];           // array for storing the different sounds 
this.$es_soundSourceFan = null;   
this.$es_fanRunning = 0;           // 0 = off, 1..16 = starting, 17 = running 

this.$es_paused = false;            // game paused

// worldscript events
this.startUp = function() 
{
    // load and initialize engine sounds. 
	for(var i=1; i<26; i++) 
	{
		this.$es_soundsEngines[i] = Sound.load("bgs-m_ambi_engine1-"+i+"n.ogg");
	}
	this.$es_soundsEngines[26] = Sound.load("bgs-m_ambi_engine1-25n_torus.ogg");    // currently a modulated version of file #25.

	this.$es_soundSourceEngines = new SoundSource;
	this.$es_soundSourceEngines.sound = this.$es_soundsEngines[1]; 
	this.$es_soundPlaying  = 1;   // actually the first sound played after launching will be somewhere in the middle range      
	this.$es_soundSourceEngines.loop = true;
	this.$es_soundSourceEngines.volume = 0;
	
	if (this.$es_torusSound)
	{	
		// initialize torus starting and stopping sounds
		this.$es_soundsTorus[1] = Sound.load("bgs-m_fx_engineup1n.ogg");    
		this.$es_soundsTorus[2] = Sound.load("bgs-m_fx_enginedown1n.ogg");
		this.$es_soundSourceTorus = new SoundSource;
		this.$es_soundSourceTorus.sound = $es_soundsTorus[1];
		this.$es_soundSourceTorus.loop = false;
		this.$es_soundSourceTorus.volume = this.$es_volume;
		this.$es_torusOn = false;
	}	

	if (this.$es_maneuveringSound > 0)
	{	
		// initialize maneuvering sounds 
		this.$es_soundSourceManeuver  = new SoundSource;
		this.$es_soundSourceManeuver.sound = Sound.load("bgs-m_ambi_engine1-"+this.$es_maneuveringSound+"n.ogg");
		this.$es_soundSourceManeuver.loop = true;
		this.$es_soundSourceManeuver.volume = this.$es_volumeManeuver; 
		// this.$es_soundSourceManeuver.play();    // play continuously, only volume will be changed
		this.$es_oldOrientation = new Quaternion();
	}	

	// initialize witch drive sounds 
	// this.$es_soundsWitch[1] = Sound.load("es_witch_start.ogg");    
	// this.$es_soundsWitch[2] = Sound.load("es_witch_stop.ogg");    
	// this.$es_soundSourceWitch = new SoundSource;
	// this.$es_soundSourceWitch.sound = $es_soundsWitch[1];
	// this.$es_soundSourceWitch.volume = this.$es_volume;
	
	if (this.$es_volumeFan > 0)
    {
		// initialize cooling fan sounds 
		this.$es_soundsFan[1] = Sound.load("es_fan_start.ogg");    
		this.$es_soundsFan[2] = Sound.load("es_fan_loop.ogg");
		this.$es_soundsFan[3] = Sound.load("es_fan_stop.ogg");    
		this.$es_soundSourceFan = new SoundSource;
		this.$es_soundSourceFan.sound = $es_soundsFan[1];
		this.$es_soundSourceFan.volume = this.$es_volumeFan;
		this.$es_fanRunning = 0;
	}		

   	// initialise timer. 0.25 is the minimal value so we need two timers 
	this.$es_timer1 =  new Timer(this, this.$es_changeSounds, 0, 0.25);        
	this.$es_timer2 =  new Timer(this, this.$es_changeSounds, 0.125, 0.25);
};

// BGS OXP ambience engine sound, accelerating and decelerating sounds must be deactivated. 
// Isn't perfect this way because BGS can't be configured to the needed detail, so 
// there will be more sounds deactivated than needed. 
this.startUpComplete = function() 
{
 	var b = worldScripts["BGS-M"];
	if(b) 
	{
		log ("Engine Sound: BGS-M detected");
		// turn off BGS ambient sounds 
		if (this.$es_BGSoff || (b.bgsDisableChatter && b.bgsDisableCrowd)) b.ambientSounds = false; 
		b.fxSounds = false;      // turns off accelerating and decelerating engine sound effects 		
	}	
 };

// timer function called every 1/8 s
this.$es_changeSounds = function() 
{
	var p = player.ship;
	if( !p || !p.isValid || p.docked || this.$es_paused) 
	{
		this.$es_soundSourceEngines.volume = 0;   
		if (this.$es_maneuveringSound > 0) this.$es_soundSourceManeuver.stop();   
		if (this.$es_volumeFan > 0) this.$es_soundSourceFan.stop();   
		return;
	}

	var pSpeed = p.speed/p.maxSpeed;  

	if (pSpeed >= 1) 
	{
		// note: speed > maxSpeed could mean injector speed too, but the injectors do have
		// their own sound in the core game, so we don't need to do anything in this case.
		// (injector speed is 7 x maxSpeed)
		if (this.$es_torusSound)
		{		
			if (p.torusEngaged && !this.$es_torusOn)
			{
				this.$es_torusOn = true;
				this.$es_soundSourceTorus.sound = this.$es_soundsTorus[1];
				this.$es_soundSourceTorus.play();
			}	
			else if (!p.torusEngaged && this.$es_torusOn) 
			{
				// torus drive stopped 
				this.$es_torusOn = false;
				this.$es_soundSourceTorus.sound = this.$es_soundsTorus[2];
				this.$es_soundSourceTorus.play();
			}	
		}		
		pSpeed = 1;    // keeps engines running at full speed 
	}	

    if (this.$es_maneuveringSound > 0)
	{		
		// Make a maneuvering sound if the orientation has changed in any way. Because of inaccuracies 
		// when comparing float values, a threshold must be specified. 
		if (Math.abs(p.orientation.x - this.$es_oldOrientation.x ) > 0.0001 || 
			Math.abs(p.orientation.y - this.$es_oldOrientation.y ) > 0.0001 || 
			Math.abs(p.orientation.z - this.$es_oldOrientation.z ) > 0.0001)
		{	
			if (!this.$es_soundSourceManeuver.isPlaying) this.$es_soundSourceManeuver.play();
			this.$es_oldOrientation = p.orientation;	
		}
		else if (this.$es_soundSourceManeuver.isPlaying) 
		{	
			this.$es_soundSourceManeuver.stop();
		}
	}

	if (this.$es_volumeFan > 0)
    {
		// cooling fan 
		if (p.temperature >= this.$es_fanStartTemp || (this.$es_fanRunning > 0 && this.$es_fanRunning < 17))
		{
			if (this.$es_fanRunning === 0)	
			{
				this.$es_soundSourceFan.sound = this.$es_soundsFan[1];
				this.$es_soundSourceFan.loop = false;
				this.$es_soundSourceFan.play();  
				this.$es_fanRunning = 1;
			}		
			else if (this.$es_fanRunning < 16)	
			{
				// startup sound is 2s long, so we have to wait 16 timer cyles before switching to loop sound
				this.$es_fanRunning++;   
			}
			else if (this.$es_fanRunning === 16)	
			{	
				this.$es_soundSourceFan.sound = this.$es_soundsFan[2];
				this.$es_soundSourceFan.loop = true;
				this.$es_soundSourceFan.play();  
				this.$es_fanRunning = 17;   
			}
		}		
		else if (p.temperature < this.$es_fanStartTemp && this.$es_fanRunning >= 17)
		{
			// this.$es_soundSourceFan.stop();
			this.$es_soundSourceFan.sound = this.$es_soundsFan[3];
			this.$es_soundSourceFan.loop = false;
			this.$es_soundSourceFan.play();
			this.$es_fanRunning = 0;
		}		
    }
	
    // To get a standby sound we use sound #1 for speed 0 and sound #2 for speeds even slightly higher than 0. 
	if (pSpeed === 0) 
	{
		var sound = 1;
		var vol = this.$es_volumeStandby;
	} 
	else if (p.torusEngaged)
	{
		var sound = 26;
		var vol = this.$es_volume;
	}
	else	
	{
		var sound = Math.ceil(pSpeed*24)+1; // [2..25]
		var vol = this.$es_volume;
    }	
	
	if(sound != this.$es_soundPlaying) 
	{
		this.$es_soundSourceEngines.sound = this.$es_soundsEngines[sound];
		this.$es_soundSourceEngines.volume = vol;
		this.$es_soundSourceEngines.play();
		this.$es_soundPlaying = sound;
	}
};

// witch space events (not ready for release yet)
// this.playerStartedJumpCountdown = function(type, seconds)
// {
// 	this.$es_soundSourceWitch.sound = this.$es_soundsWitch[1];
//	this.$es_soundSourceWitch.play();     
// };
// this.shipWillEnterWitchspace = function(cause, destination)
// {
// 	this.$es_soundSourceWitch.sound = this.$es_soundsWitch[2];
//	this.$es_soundSourceWitch.play();     
// };
// this.playerCancelledJumpCountdown = function()
// {
	// this will be the difficult part because the spinning up sould would have to be interrupted 
	// and follwed by a spinning down sound starting at the correct frequency.
// }

// play system shutdown sound during docking
this.shipWillDockWithStation = function(station)
{
   if (this.$es_shutOff)
   {   
		this.$es_soundSourceTorus.sound = this.$es_soundsTorus[2];
		this.$es_soundSourceTorus.play();
   }	
};

this.gamePaused = function() 
{
	this.$es_soundSourceEngines.stop();   
    if (this.$es_maneuveringSound > 0) this.$es_soundSourceManeuver.stop();   
	if (this.$es_volumeFan > 0) this.$es_soundSourceFan.stop();   
	this.$es_paused = true;
};

this.gameResumed = function() 
{
	this.$es_soundSourceEngines.play();   
	if (this.$es_volumeFan > 0) this.$es_soundSourceFan.play();   
	// restarting is not needed for maneuvering sound
	this.$es_paused = false;
};
